@@ -65,7 +65,6 @@ static int value_count(grib_accessor*, long*);
 static void destroy(grib_context*, grib_accessor*);
 static void dump(grib_accessor*, grib_dumper*);
 static void init(grib_accessor*, const long, grib_arguments*);
-static void init_class(grib_accessor_class*);
 static int unpack_double_element(grib_accessor*, size_t i, double* val);
 static grib_accessor* make_clone(grib_accessor*, grib_section*, int*);
 
@@ -93,32 +92,32 @@ static grib_accessor_class _grib_accessor_class_bufr_data_element = {
     "bufr_data_element",                      /* name */
     sizeof(grib_accessor_bufr_data_element),  /* size */
     0,                           /* inited */
-    &init_class,                 /* init_class */
+    0,                           /* init_class */
     &init,                       /* init */
     0,                  /* post_init */
-    &destroy,                    /* free mem */
-    &dump,                       /* describes himself */
-    0,                /* get length of section */
+    &destroy,                    /* destroy */
+    &dump,                       /* dump */
+    0,                /* next_offset */
     0,              /* get length of string */
     &value_count,                /* get number of values */
     0,                 /* get number of bytes */
     0,                /* get offset to bytes */
     &get_native_type,            /* get native type */
     0,                /* get sub_section */
-    &pack_missing,               /* grib_pack procedures long */
-    &is_missing,                 /* grib_pack procedures long */
-    &pack_long,                  /* grib_pack procedures long */
-    &unpack_long,                /* grib_unpack procedures long */
-    &pack_double,                /* grib_pack procedures double */
-    0,                 /* grib_pack procedures float */
-    &unpack_double,              /* grib_unpack procedures double */
-    0,               /* grib_unpack procedures float */
-    &pack_string,                /* grib_pack procedures string */
-    &unpack_string,              /* grib_unpack procedures string */
-    &pack_string_array,          /* grib_pack array procedures string */
-    &unpack_string_array,        /* grib_unpack array procedures string */
-    0,                 /* grib_pack procedures bytes */
-    0,               /* grib_unpack procedures bytes */
+    &pack_missing,               /* pack_missing */
+    &is_missing,                 /* is_missing */
+    &pack_long,                  /* pack_long */
+    &unpack_long,                /* unpack_long */
+    &pack_double,                /* pack_double */
+    0,                 /* pack_float */
+    &unpack_double,              /* unpack_double */
+    0,               /* unpack_float */
+    &pack_string,                /* pack_string */
+    &unpack_string,              /* unpack_string */
+    &pack_string_array,          /* pack_string_array */
+    &unpack_string_array,        /* unpack_string_array */
+    0,                 /* pack_bytes */
+    0,               /* unpack_bytes */
     0,            /* pack_expression */
     0,              /* notify_change */
     0,                /* update_size */
@@ -127,10 +126,10 @@ static grib_accessor_class _grib_accessor_class_bufr_data_element = {
     0,      /* nearest_smaller_value */
     0,                       /* next accessor */
     0,                    /* compare vs. another accessor */
-    &unpack_double_element,      /* unpack only ith value */
-    0,       /* unpack only ith value */
-    0,  /* unpack a given set of elements */
-    0,   /* unpack a given set of elements */
+    &unpack_double_element,      /* unpack only ith value (double) */
+    0,       /* unpack only ith value (float) */
+    0,  /* unpack a given set of elements (double) */
+    0,   /* unpack a given set of elements (float) */
     0,     /* unpack a subarray */
     0,                      /* clear */
     &make_clone,                 /* clone accessor */
@@ -139,33 +138,6 @@ static grib_accessor_class _grib_accessor_class_bufr_data_element = {
 
 grib_accessor_class* grib_accessor_class_bufr_data_element = &_grib_accessor_class_bufr_data_element;
 
-
-static void init_class(grib_accessor_class* c)
-{
-    c->next_offset    =    (*(c->super))->next_offset;
-    c->string_length    =    (*(c->super))->string_length;
-    c->byte_count    =    (*(c->super))->byte_count;
-    c->byte_offset    =    (*(c->super))->byte_offset;
-    c->sub_section    =    (*(c->super))->sub_section;
-    c->pack_float    =    (*(c->super))->pack_float;
-    c->unpack_float    =    (*(c->super))->unpack_float;
-    c->pack_bytes    =    (*(c->super))->pack_bytes;
-    c->unpack_bytes    =    (*(c->super))->unpack_bytes;
-    c->pack_expression    =    (*(c->super))->pack_expression;
-    c->notify_change    =    (*(c->super))->notify_change;
-    c->update_size    =    (*(c->super))->update_size;
-    c->preferred_size    =    (*(c->super))->preferred_size;
-    c->resize    =    (*(c->super))->resize;
-    c->nearest_smaller_value    =    (*(c->super))->nearest_smaller_value;
-    c->next    =    (*(c->super))->next;
-    c->compare    =    (*(c->super))->compare;
-    c->unpack_float_element    =    (*(c->super))->unpack_float_element;
-    c->unpack_double_element_set    =    (*(c->super))->unpack_double_element_set;
-    c->unpack_float_element_set    =    (*(c->super))->unpack_float_element_set;
-    c->unpack_double_subarray    =    (*(c->super))->unpack_double_subarray;
-    c->clear    =    (*(c->super))->clear;
-}
-
 /* END_CLASS_IMP */
 
 static grib_accessor* make_clone(grib_accessor* a, grib_section* s, int* err)
@@ -219,61 +191,61 @@ static grib_accessor* make_clone(grib_accessor* a, grib_section* s, int* err)
 void accessor_bufr_data_element_set_index(grib_accessor* a, long index)
 {
     grib_accessor_bufr_data_element* self = (grib_accessor_bufr_data_element*)a;
-    self->index                           = index;
+    self->index = index;
 }
 
 void accessor_bufr_data_element_set_type(grib_accessor* a, int type)
 {
     grib_accessor_bufr_data_element* self = (grib_accessor_bufr_data_element*)a;
-    self->type                            = type;
+    self->type = type;
 }
 
 void accessor_bufr_data_element_set_numberOfSubsets(grib_accessor* a, long numberOfSubsets)
 {
     grib_accessor_bufr_data_element* self = (grib_accessor_bufr_data_element*)a;
-    self->numberOfSubsets                 = numberOfSubsets;
+    self->numberOfSubsets = numberOfSubsets;
 }
 
 void accessor_bufr_data_element_set_subsetNumber(grib_accessor* a, long subsetNumber)
 {
     grib_accessor_bufr_data_element* self = (grib_accessor_bufr_data_element*)a;
-    self->subsetNumber                    = subsetNumber;
+    self->subsetNumber = subsetNumber;
 }
 
 void accessor_bufr_data_element_set_compressedData(grib_accessor* a, int compressedData)
 {
     grib_accessor_bufr_data_element* self = (grib_accessor_bufr_data_element*)a;
-    self->compressedData                  = compressedData;
+    self->compressedData = compressedData;
 }
 
 void accessor_bufr_data_element_set_descriptors(grib_accessor* a, bufr_descriptors_array* descriptors)
 {
     grib_accessor_bufr_data_element* self = (grib_accessor_bufr_data_element*)a;
-    self->descriptors                     = descriptors;
+    self->descriptors = descriptors;
 }
 
 void accessor_bufr_data_element_set_numericValues(grib_accessor* a, grib_vdarray* numericValues)
 {
     grib_accessor_bufr_data_element* self = (grib_accessor_bufr_data_element*)a;
-    self->numericValues                   = numericValues;
+    self->numericValues = numericValues;
 }
 
 void accessor_bufr_data_element_set_stringValues(grib_accessor* a, grib_vsarray* stringValues)
 {
     grib_accessor_bufr_data_element* self = (grib_accessor_bufr_data_element*)a;
-    self->stringValues                    = stringValues;
+    self->stringValues = stringValues;
 }
 
 void accessor_bufr_data_element_set_elementsDescriptorsIndex(grib_accessor* a, grib_viarray* elementsDescriptorsIndex)
 {
     grib_accessor_bufr_data_element* self = (grib_accessor_bufr_data_element*)a;
-    self->elementsDescriptorsIndex        = elementsDescriptorsIndex;
+    self->elementsDescriptorsIndex = elementsDescriptorsIndex;
 }
 
 static void init(grib_accessor* a, const long len, grib_arguments* params)
 {
     grib_accessor_bufr_data_element* self = (grib_accessor_bufr_data_element*)a;
-    a->length                             = 0;
+    a->length = 0;
     a->flags |= GRIB_ACCESSOR_FLAG_BUFR_DATA;
     /* a->flags |= GRIB_ACCESSOR_FLAG_READ_ONLY; */
     self->cname = NULL;
@@ -300,14 +272,14 @@ static int unpack_string_array(grib_accessor* a, char** val, size_t* len)
 {
     grib_accessor_bufr_data_element* self = (grib_accessor_bufr_data_element*)a;
 
-    int ret         = 0, i, idx;
-    long count      = 0;
+    int ret = 0, idx = 0;
+    size_t count = 0, i = 0;
     grib_context* c = a->context;
 
     if (self->compressedData) {
-        DebugAssert(self->index < self->numericValues->n);
+        DEBUG_ASSERT(self->index < self->numericValues->n);
         idx = ((int)self->numericValues->v[self->index]->v[0] / 1000 - 1) / self->numberOfSubsets;
-        DebugAssert(idx < self->stringValues->n);
+        DEBUG_ASSERT(idx < self->stringValues->n);
         count = grib_sarray_used_size(self->stringValues->v[idx]);
         for (i = 0; i < count; i++) {
             val[i] = grib_context_strdup(c, self->stringValues->v[idx]->v[i]);
@@ -315,9 +287,9 @@ static int unpack_string_array(grib_accessor* a, char** val, size_t* len)
         *len = count;
     }
     else {
-        DebugAssert(self->subsetNumber < self->numericValues->n);
-        DebugAssert(self->index < self->numericValues->v[self->subsetNumber]->n);
-        idx    = (int)self->numericValues->v[self->subsetNumber]->v[self->index] / 1000 - 1;
+        DEBUG_ASSERT(self->subsetNumber < self->numericValues->n);
+        DEBUG_ASSERT(self->index < self->numericValues->v[self->subsetNumber]->n);
+        idx = (int)self->numericValues->v[self->subsetNumber]->v[self->index] / 1000 - 1;
         val[0] = grib_context_strdup(c, self->stringValues->v[idx]->v[0]);
         *len   = 1;
     }
@@ -336,7 +308,7 @@ static int pack_string_array(grib_accessor* a, const char** v, size_t* len)
 
     if (self->compressedData) {
         idx = ((int)self->numericValues->v[self->index]->v[0] / 1000 - 1) / self->numberOfSubsets;
-        if (*len != 1 && *len != self->numberOfSubsets) {
+        if (*len != 1 && *len != (size_t)self->numberOfSubsets) {
             grib_context_log(c, GRIB_LOG_ERROR, "Number of values mismatch for '%s': %ld strings provided but expected %ld (=number of subsets)",
                              self->descriptors->v[self->elementsDescriptorsIndex->v[0]->v[idx]]->shortName, *len, self->numberOfSubsets);
             return GRIB_ARRAY_TOO_SMALL;
@@ -350,7 +322,19 @@ static int pack_string_array(grib_accessor* a, const char** v, size_t* len)
         }
     }
     else {
-        ret = GRIB_NOT_IMPLEMENTED;
+        //ECC-1623
+        if (*len != (size_t)self->numberOfSubsets) {
+            grib_context_log(c, GRIB_LOG_ERROR,
+                "Number of values mismatch for '%s': %zu strings provided but expected %ld (=number of subsets)",
+                a->name, *len, self->numberOfSubsets);
+            return GRIB_WRONG_ARRAY_SIZE;
+        }
+        for (i = 0; i < *len; i++) {
+            //idx = (int)self->numericValues->v[self->subsetNumber]->v[self->index] / 1000 - 1;
+            idx = (int)self->numericValues->v[i]->v[self->index] / 1000 - 1;
+            self->stringValues->v[idx]->v[0] = strdup(v[i]);
+        }
+        *len=1;
     }
 
     return ret;
@@ -359,8 +343,8 @@ static int pack_string_array(grib_accessor* a, const char** v, size_t* len)
 static int unpack_string(grib_accessor* a, char* val, size_t* len)
 {
     grib_accessor_bufr_data_element* self = (grib_accessor_bufr_data_element*)a;
-    char* str   = NULL;
-    char* p     = 0;
+    char* str = NULL;
+    char* p = 0;
     size_t slen = 0;
     double dval = 0;
     size_t dlen = 1;
@@ -374,24 +358,24 @@ static int unpack_string(grib_accessor* a, char* val, size_t* len)
         snprintf(sval, sizeof(sval), "%g", dval);
         slen = strlen(sval);
         if (*len < slen)
-            return GRIB_ARRAY_TOO_SMALL;
+            return GRIB_BUFFER_TOO_SMALL;
         strcpy(val, sval);
         return GRIB_SUCCESS;
     }
 
     if (self->compressedData) {
-        DebugAssert(self->index < self->numericValues->n);
+        DEBUG_ASSERT(self->index < self->numericValues->n);
         idx = ((int)self->numericValues->v[self->index]->v[0] / 1000 - 1) / self->numberOfSubsets;
         if (idx < 0)
             return GRIB_INTERNAL_ERROR;
         str = grib_context_strdup(c, self->stringValues->v[idx]->v[0]);
     }
     else {
-        DebugAssert(self->subsetNumber < self->numericValues->n);
+        DEBUG_ASSERT(self->subsetNumber < self->numericValues->n);
         idx = (int)self->numericValues->v[self->subsetNumber]->v[self->index] / 1000 - 1;
         if (idx < 0)
             return GRIB_INTERNAL_ERROR;
-        DebugAssert(idx < self->stringValues->n);
+        DEBUG_ASSERT(idx < self->stringValues->n);
         str = grib_context_strdup(c, self->stringValues->v[idx]->v[0]);
     }
 
@@ -429,8 +413,8 @@ static int pack_string(grib_accessor* a, const char* val, size_t* len)
 {
     grib_accessor_bufr_data_element* self = (grib_accessor_bufr_data_element*)a;
 
-    int ret         = GRIB_SUCCESS, idx = 0;
-    char* s         = NULL;
+    int ret = GRIB_SUCCESS, idx = 0;
+    char* s = NULL;
     grib_context* c = a->context;
 
     if (self->compressedData) {
@@ -442,7 +426,7 @@ static int pack_string(grib_accessor* a, const char* val, size_t* len)
     grib_sarray_delete_content(c, self->stringValues->v[idx]); /* ECC-1172 */
     grib_sarray_delete(c, self->stringValues->v[idx]);
     self->stringValues->v[idx] = grib_sarray_new(c, 1, 1);
-    s                          = grib_context_strdup(c, val);
+    s = grib_context_strdup(c, val);
     grib_sarray_push(c, self->stringValues->v[idx], s);
 
     return ret;
@@ -451,7 +435,7 @@ static int pack_string(grib_accessor* a, const char* val, size_t* len)
 static int unpack_long(grib_accessor* a, long* val, size_t* len)
 {
     grib_accessor_bufr_data_element* self = (grib_accessor_bufr_data_element*)a;
-    int ret    = GRIB_SUCCESS;
+    int ret = GRIB_SUCCESS;
     long count = 0, i = 0;
 
     value_count(a, &count);
@@ -461,15 +445,15 @@ static int unpack_long(grib_accessor* a, long* val, size_t* len)
 
     if (self->compressedData) {
         for (i = 0; i < count; i++) {
-            DebugAssert(self->index < self->numericValues->n);
-            DebugAssert(i < self->numericValues->v[self->index]->n);
+            DEBUG_ASSERT(self->index < self->numericValues->n);
+            DEBUG_ASSERT(i < self->numericValues->v[self->index]->n);
             val[i] = self->numericValues->v[self->index]->v[i] == GRIB_MISSING_DOUBLE ? GRIB_MISSING_LONG : (long)self->numericValues->v[self->index]->v[i];
         }
         *len = count;
     }
     else {
-        DebugAssert(self->subsetNumber < self->numericValues->n);
-        DebugAssert(self->index < self->numericValues->v[self->subsetNumber]->n);
+        DEBUG_ASSERT(self->subsetNumber < self->numericValues->n);
+        DEBUG_ASSERT(self->index < self->numericValues->v[self->subsetNumber]->n);
         val[0] = self->numericValues->v[self->subsetNumber]->v[self->index] == GRIB_MISSING_DOUBLE ? GRIB_MISSING_LONG : (long)self->numericValues->v[self->subsetNumber]->v[self->index];
         *len   = 1;
     }
@@ -490,15 +474,15 @@ static int unpack_double(grib_accessor* a, double* val, size_t* len)
 
     if (self->compressedData) {
         for (i = 0; i < count; i++) {
-            DebugAssert(self->index < self->numericValues->n);
-            DebugAssert(i < self->numericValues->v[self->index]->n);
+            DEBUG_ASSERT(self->index < self->numericValues->n);
+            DEBUG_ASSERT(i < self->numericValues->v[self->index]->n);
             val[i] = self->numericValues->v[self->index]->v[i];
         }
         *len = count;
     }
     else {
-        DebugAssert(self->subsetNumber < self->numericValues->n);
-        DebugAssert(self->index < self->numericValues->v[self->subsetNumber]->n);
+        DEBUG_ASSERT(self->subsetNumber < self->numericValues->n);
+        DEBUG_ASSERT(self->index < self->numericValues->v[self->subsetNumber]->n);
         val[0] = self->numericValues->v[self->subsetNumber]->v[self->index];
         *len   = 1;
     }
@@ -510,12 +494,12 @@ static int pack_double(grib_accessor* a, const double* val, size_t* len)
 {
     grib_accessor_bufr_data_element* self = (grib_accessor_bufr_data_element*)a;
     int ret = GRIB_SUCCESS;
-    long count = 1, i = 0;
+    size_t count = 1, i = 0;
     grib_context* c = a->context;
 
     if (self->compressedData) {
         count = *len;
-        if (count != 1 && count != self->numberOfSubsets) {
+        if (count != 1 && count != (size_t)self->numberOfSubsets) {
             grib_context_log(c, GRIB_LOG_ERROR, "Number of values mismatch for '%s': %ld doubles provided but expected %ld (=number of subsets)",
                              self->descriptors->v[self->elementsDescriptorsIndex->v[0]->v[self->index]]->shortName, count, self->numberOfSubsets);
             return GRIB_ARRAY_TOO_SMALL;
@@ -530,7 +514,7 @@ static int pack_double(grib_accessor* a, const double* val, size_t* len)
     }
     else {
         self->numericValues->v[self->subsetNumber]->v[self->index] = val[0];
-        *len                                                       = 1;
+        *len = 1;
     }
 
     return ret;
@@ -539,14 +523,14 @@ static int pack_double(grib_accessor* a, const double* val, size_t* len)
 static int pack_long(grib_accessor* a, const long* val, size_t* len)
 {
     grib_accessor_bufr_data_element* self = (grib_accessor_bufr_data_element*)a;
-    int ret                               = 0, i;
-    long count                            = 1;
-    grib_context* c                       = a->context;
+    int ret = 0;
+    size_t count = 1, i = 0;
+    grib_context* c = a->context;
 
     if (self->compressedData) {
         count = *len;
-        if (count != 1 && count != self->numberOfSubsets) {
-            grib_context_log(c, GRIB_LOG_ERROR, "Number of values mismatch for '%s': %ld integers provided but expected %ld (=number of subsets)",
+        if (count != 1 && count != (size_t)self->numberOfSubsets) {
+            grib_context_log(c, GRIB_LOG_ERROR, "Number of values mismatch for '%s': %zu integers provided but expected %ld (=number of subsets)",
                              self->descriptors->v[self->elementsDescriptorsIndex->v[0]->v[self->index]]->shortName, count, self->numberOfSubsets);
             return GRIB_ARRAY_TOO_SMALL;
         }
@@ -560,7 +544,7 @@ static int pack_long(grib_accessor* a, const long* val, size_t* len)
     }
     else {
         self->numericValues->v[self->subsetNumber]->v[self->index] = val[0] == GRIB_MISSING_LONG ? GRIB_MISSING_DOUBLE : val[0];
-        *len                                                       = 1;
+        *len = 1;
     }
 
     return ret;
@@ -568,7 +552,7 @@ static int pack_long(grib_accessor* a, const long* val, size_t* len)
 
 static int value_count(grib_accessor* a, long* count)
 {
-    int ret     = 0, type = 0, idx = 0;
+    int ret = 0, type = 0, idx = 0;
     size_t size = 0;
     grib_accessor_bufr_data_element* self = (grib_accessor_bufr_data_element*)a;
 
@@ -579,12 +563,12 @@ static int value_count(grib_accessor* a, long* count)
     type = get_native_type(a);
 
     if (type == GRIB_TYPE_STRING) {
-        DebugAssert(self->index < self->numericValues->n);
+        DEBUG_ASSERT(self->index < self->numericValues->n);
         idx  = ((int)self->numericValues->v[self->index]->v[0] / 1000 - 1) / self->numberOfSubsets;
         size = grib_sarray_used_size(self->stringValues->v[idx]);
     }
     else {
-        DebugAssert(self->index < self->numericValues->n);
+        DEBUG_ASSERT(self->index < self->numericValues->n);
         size = grib_darray_used_size(self->numericValues->v[self->index]);
     }
 
@@ -597,8 +581,8 @@ static int unpack_double_element(grib_accessor* a, size_t idx, double* val)
 {
     /* ECC-415 */
     grib_accessor_bufr_data_element* self = (grib_accessor_bufr_data_element*)a;
-    int ret       = GRIB_SUCCESS;
-    long count    = 0;
+    int ret = GRIB_SUCCESS;
+    long count = 0;
 
     value_count(a, &count);
     if (idx >= (size_t)count) {
@@ -617,8 +601,8 @@ static int unpack_double_element(grib_accessor* a, size_t idx, double* val)
 static int get_native_type(grib_accessor* a)
 {
     grib_accessor_bufr_data_element* self = (grib_accessor_bufr_data_element*)a;
-    int ret                               = GRIB_TYPE_DOUBLE;
-    DebugAssert(self);
+    int ret = GRIB_TYPE_DOUBLE;
+    DEBUG_ASSERT(self);
     switch (self->type) {
         case BUFR_DESCRIPTOR_TYPE_STRING:
             ret = GRIB_TYPE_STRING;
@@ -643,7 +627,7 @@ static int get_native_type(grib_accessor* a)
 static void destroy(grib_context* ct, grib_accessor* a)
 {
     grib_accessor_bufr_data_element* self = (grib_accessor_bufr_data_element*)a;
-    int i                                 = 0;
+    int i = 0;
     if (self->cname)
         grib_context_free(ct, self->cname); /* ECC-765 */
     while (i < MAX_ACCESSOR_ATTRIBUTES && a->attributes[i]) {
@@ -666,14 +650,14 @@ static int is_missing(grib_accessor* a)
     grib_context* c = a->context;
 
     if (ktype == GRIB_TYPE_LONG) {
-        long* values           = NULL;
-        long value             = 0;
+        long* values = NULL;
+        long value = 0;
 
         value_count(a, &count);
         size = size2 = count;
         if (size > 1) {
             values = (long*)grib_context_malloc_clear(c, sizeof(long) * size);
-            err    = grib_unpack_long(a, values, &size2);
+            err = grib_unpack_long(a, values, &size2);
         }
         else {
             err = grib_unpack_long(a, &value, &size2);
@@ -693,14 +677,14 @@ static int is_missing(grib_accessor* a)
         }
     }
     else if (ktype == GRIB_TYPE_DOUBLE) {
-        double value           = 0;
-        double* values         = NULL;
+        double value = 0;
+        double* values = NULL;
 
         value_count(a, &count);
         size = size2 = count;
         if (size > 1) {
             values = (double*)grib_context_malloc_clear(c, sizeof(double) * size);
-            err    = grib_unpack_double(a, values, &size2);
+            err = grib_unpack_double(a, values, &size2);
         }
         else {
             err = grib_unpack_double(a, &value, &size2);
@@ -721,7 +705,7 @@ static int is_missing(grib_accessor* a)
         }
     }
     else if (ktype == GRIB_TYPE_STRING) {
-        char** values          = NULL;
+        char** values = NULL;
         value_count(a, &count);
         size = count;
         if (size > 1) {
