@@ -116,26 +116,31 @@ static void dump(grib_accessor* a, grib_dumper* dumper)
 
 static int unpack_string(grib_accessor* a, char* v, size_t* len)
 {
-    int err  = 0;
     long val = 0;
     size_t l = 1;
     char repres[1024];
+    char format[32] = "%ld";
+    grib_handle* h = grib_handle_of_accessor(a);
 
-    err = grib_unpack_long(a, &val, &l);
+    grib_unpack_long(a, &val, &l);
     /* TODO: We should catch all errors but in this case the test ERA_Gen.sh will fail
      * as the output from grib_ls will be different */
-    /* if (err) return err; */
-    (void)err;
 
-    if ((val == GRIB_MISSING_LONG) && ((a->flags & GRIB_ACCESSOR_FLAG_CAN_BE_MISSING) != 0))
+    if ((val == GRIB_MISSING_LONG) && ((a->flags & GRIB_ACCESSOR_FLAG_CAN_BE_MISSING) != 0)) {
         snprintf(repres, sizeof(repres), "MISSING");
-    else
-        snprintf(repres, sizeof(repres), "%ld", val);
+    } else {
+        size_t size = sizeof(format);
+        grib_get_string(h, "formatForLongs", format, &size);
+        snprintf(repres, sizeof(repres), format, val);
+    }
 
     l = strlen(repres) + 1;
 
-    if (l > *len) {
-        grib_context_log(a->context, GRIB_LOG_ERROR, "grib_accessor_long : unpack_string : Buffer too small for %s ", a->name);
+    if (*len < l) {
+        const char* cclass_name = a->cclass->name;
+        grib_context_log(a->context, GRIB_LOG_ERROR,
+                         "%s: Buffer too small for %s. It is %zu bytes long (len=%zu)",
+                         cclass_name, a->name, l, *len);
         *len = l;
         return GRIB_BUFFER_TOO_SMALL;
     }
@@ -236,10 +241,8 @@ static int compare(grib_accessor* a, grib_accessor* b)
     grib_unpack_long(b, bval, &blen);
 
     retval = GRIB_SUCCESS;
-    while (alen != 0) {
-        if (*bval != *aval)
-            retval = GRIB_LONG_VALUE_MISMATCH;
-        alen--;
+    for (size_t i=0; i<alen && retval == GRIB_SUCCESS; ++i) {
+        if (aval[i] != bval[i]) retval = GRIB_LONG_VALUE_MISMATCH;
     }
 
     grib_context_free(a->context, aval);
@@ -252,12 +255,12 @@ static int pack_string(grib_accessor* a, const char* val, size_t* len)
 {
     long v = 0; /* The converted value */
 
-    // Requires more work e.g. filter
-    //if (strcmp_nocase(val, "missing")==0) {
-    //    return pack_missing(a);
-    //}
+    // ECC-1722
+    if (STR_EQUAL_NOCASE(val, "missing")) {
+        return pack_missing(a);
+    }
 
-    if (string_to_long(val, &v) != GRIB_SUCCESS) {
+    if (string_to_long(val, &v, 1) != GRIB_SUCCESS) {
         grib_context_log(a->context, GRIB_LOG_ERROR,
                 "Trying to pack \"%s\" as long. String cannot be converted to an integer", val);
         return GRIB_WRONG_TYPE;
