@@ -83,8 +83,7 @@ static void* default_long_lasting_malloc(const grib_context* c, size_t size)
     void* ret;
     ret = malloc(size);
     if (!ret) {
-        grib_context_log(c, GRIB_LOG_FATAL, "default_long_lasting_malloc: error allocating %zu bytes", size);
-        Assert(0);
+        grib_context_log(c, GRIB_LOG_FATAL, "%s: error allocating %zu bytes", __func__, size);
     }
     return ret;
 }
@@ -99,8 +98,7 @@ static void* default_buffer_malloc(const grib_context* c, size_t size)
     void* ret;
     ret = malloc(size);
     if (!ret) {
-        grib_context_log(c, GRIB_LOG_FATAL, "default_buffer_malloc: error allocating %zu bytes", size);
-        Assert(0);
+        grib_context_log(c, GRIB_LOG_FATAL, "%s: error allocating %zu bytes", __func__, size);
     }
     return ret;
 }
@@ -110,8 +108,7 @@ static void* default_buffer_realloc(const grib_context* c, void* p, size_t size)
     void* ret;
     ret = realloc(p, size);
     if (!ret) {
-        grib_context_log(c, GRIB_LOG_FATAL, "default_buffer_realloc: error allocating %zu bytes", size);
-        Assert(0);
+        grib_context_log(c, GRIB_LOG_FATAL, "%s: error allocating %zu bytes", __func__, size);
     }
     return ret;
 }
@@ -126,8 +123,7 @@ static void* default_malloc(const grib_context* c, size_t size)
     void* ret;
     ret = malloc(size);
     if (!ret) {
-        grib_context_log(c, GRIB_LOG_FATAL, "default_malloc: error allocating %zu bytes", size);
-        Assert(0);
+        grib_context_log(c, GRIB_LOG_FATAL, "%s: error allocating %zu bytes", __func__, size);
     }
     return ret;
 }
@@ -137,8 +133,7 @@ static void* default_realloc(const grib_context* c, void* p, size_t size)
     void* ret;
     ret = realloc(p, size);
     if (!ret) {
-        grib_context_log(c, GRIB_LOG_FATAL, "default_realloc: error allocating %zu bytes", size);
-        Assert(0);
+        grib_context_log(c, GRIB_LOG_FATAL, "%s: error allocating %zu bytes", __func__, size);
     }
     return ret;
 }
@@ -292,12 +287,13 @@ static grib_context default_grib_context = {
     0,               /* no_big_group_split         */
     0,               /* no_spd                     */
     0,               /* keep_matrix                */
+    0,               /* show_hour_stepunit         */
     0,               /* grib_definition_files_path */
     0,               /* grib_samples_path          */
     0,               /* grib_concept_path          */
     0,               /* grib_reader                */
     0,               /* user data                  */
-    GRIB_REAL_MODE8, /* real mode for fortran      */
+    CODES_REAL_MODE8, /* real mode for fortran     */
 
 #if MANAGE_MEM
     &grib_transient_free,    /* free_mem                   */
@@ -350,25 +346,26 @@ static grib_context default_grib_context = {
     0,              /* keys_count                 */
     0,              /* concepts_index             */
     0,              /* concepts_count             */
-    {0,}, /* concepts                   */
-    0, /* hash_array_index           */
-    0, /* hash_array_count           */
-    {0,},                                 /* hash_array                 */
-    0,                                 /* def_files                  */
-    0,                                 /* blocklist                  */
-    0,                                 /* ieee_packing               */
-    0,                                 /* bufrdc_mode                */
-    0,                                 /* bufr_set_to_missing_if_out_of_range */
-    0,                                 /* bufr_multi_element_constant_arrays */
-    0,                                 /* grib_data_quality_checks   */
-    0,                                 /* log_stream                 */
-    0,                                 /* classes                    */
-    0,                                 /* lists                      */
-    0,                                 /* expanded_descriptors       */
+    {0,},           /* concepts                   */
+    0,              /* hash_array_index           */
+    0,              /* hash_array_count           */
+    {0,},           /* hash_array                 */
+    0,              /* def_files                  */
+    0,              /* blocklist                  */
+    0,              /* ieee_packing               */
+    0,              /* bufrdc_mode                */
+    0,              /* bufr_set_to_missing_if_out_of_range */
+    0,              /* bufr_multi_element_constant_arrays */
+    0,              /* grib_data_quality_checks   */
+    0,              /* single_precision           */
+    0,              /* log_stream                 */
+    0,              /* classes                    */
+    0,              /* lists                      */
+    0,              /* expanded_descriptors       */
     DEFAULT_FILE_POOL_MAX_OPENED_FILES /* file_pool_max_opened_files */
 #if GRIB_PTHREADS
     ,
-    PTHREAD_MUTEX_INITIALIZER /* mutex                      */
+    PTHREAD_MUTEX_INITIALIZER /* mutex */
 #endif
 };
 
@@ -392,10 +389,12 @@ grib_context* grib_context_get_default()
         const char* no_big_group_split                  = NULL;
         const char* no_spd                              = NULL;
         const char* keep_matrix                         = NULL;
+        const char* show_hour_stepunit                  = NULL;
         const char* bufrdc_mode                         = NULL;
         const char* bufr_set_to_missing_if_out_of_range = NULL;
         const char* bufr_multi_element_constant_arrays  = NULL;
         const char* grib_data_quality_checks            = NULL;
+        const char* single_precision                    = NULL;
         const char* file_pool_max_opened_files          = NULL;
 
 #ifdef ENABLE_FLOATING_POINT_EXCEPTIONS
@@ -407,6 +406,7 @@ grib_context* grib_context_get_default()
         bufr_set_to_missing_if_out_of_range = getenv("ECCODES_BUFR_SET_TO_MISSING_IF_OUT_OF_RANGE");
         bufr_multi_element_constant_arrays  = getenv("ECCODES_BUFR_MULTI_ELEMENT_CONSTANT_ARRAYS");
         grib_data_quality_checks            = getenv("ECCODES_GRIB_DATA_QUALITY_CHECKS");
+        single_precision                    = getenv("ECCODES_SINGLE_PRECISION");
         large_constant_fields               = codes_getenv("ECCODES_GRIB_LARGE_CONSTANT_FIELDS");
         no_abort                            = codes_getenv("ECCODES_NO_ABORT");
         debug                               = codes_getenv("ECCODES_DEBUG");
@@ -417,6 +417,7 @@ grib_context* grib_context_get_default()
         no_big_group_split                  = codes_getenv("ECCODES_GRIB_NO_BIG_GROUP_SPLIT");
         no_spd                              = codes_getenv("ECCODES_GRIB_NO_SPD");
         keep_matrix                         = codes_getenv("ECCODES_GRIB_KEEP_MATRIX");
+        show_hour_stepunit                  = codes_getenv("ECCODES_GRIB_HOURLY_STEPS_WITH_UNITS");
         file_pool_max_opened_files          = getenv("ECCODES_FILE_POOL_MAX_OPENED_FILES");
 
         /* On UNIX, when we read from a file we get exactly what is in the file on disk.
@@ -431,6 +432,7 @@ grib_context* grib_context_get_default()
         default_grib_context.no_big_group_split = no_big_group_split ? atoi(no_big_group_split) : 0;
         default_grib_context.no_spd = no_spd ? atoi(no_spd) : 0;
         default_grib_context.keep_matrix = keep_matrix ? atoi(keep_matrix) : 1;
+        default_grib_context.grib_hourly_steps_with_units = show_hour_stepunit ? atoi(show_hour_stepunit) : 0;
         default_grib_context.write_on_fail = write_on_fail ? atoi(write_on_fail) : 0;
         default_grib_context.no_abort = no_abort ? atoi(no_abort) : 0;
         default_grib_context.debug = debug ? atoi(debug) : 0;
@@ -451,7 +453,7 @@ grib_context* grib_context_get_default()
 
 #ifdef ECCODES_SAMPLES_PATH
         if (!default_grib_context.grib_samples_path)
-            default_grib_context.grib_samples_path = ECCODES_SAMPLES_PATH;
+            default_grib_context.grib_samples_path = (char*)ECCODES_SAMPLES_PATH;
 #endif
 
         default_grib_context.grib_definition_files_path = codes_getenv("ECCODES_DEFINITION_PATH");
@@ -533,11 +535,10 @@ grib_context* grib_context_get_default()
         }
 #endif
 
-        grib_context_log(&default_grib_context, GRIB_LOG_DEBUG, "Definitions path: %s",
-                         default_grib_context.grib_definition_files_path);
-        grib_context_log(&default_grib_context, GRIB_LOG_DEBUG, "Samples path:     %s",
-                         default_grib_context.grib_samples_path);
-
+        if (default_grib_context.debug) {
+            fprintf(stderr, "ECCODES DEBUG Definitions path: %s\n", default_grib_context.grib_definition_files_path);
+            fprintf(stderr, "ECCODES DEBUG Samples path:     %s\n", default_grib_context.grib_samples_path);
+        }
         default_grib_context.keys_count = 0;
         default_grib_context.keys       = grib_hash_keys_new(&(default_grib_context), &(default_grib_context.keys_count));
 
@@ -550,6 +551,7 @@ grib_context* grib_context_get_default()
         default_grib_context.bufr_set_to_missing_if_out_of_range = bufr_set_to_missing_if_out_of_range ? atoi(bufr_set_to_missing_if_out_of_range) : 0;
         default_grib_context.bufr_multi_element_constant_arrays = bufr_multi_element_constant_arrays ? atoi(bufr_multi_element_constant_arrays) : 0;
         default_grib_context.grib_data_quality_checks = grib_data_quality_checks ? atoi(grib_data_quality_checks) : 0;
+        default_grib_context.single_precision = single_precision ? atoi(single_precision) : 0;
         default_grib_context.file_pool_max_opened_files = file_pool_max_opened_files ? atoi(file_pool_max_opened_files) : DEFAULT_FILE_POOL_MAX_OPENED_FILES;
     }
 
@@ -557,52 +559,52 @@ grib_context* grib_context_get_default()
     return &default_grib_context;
 }
 
-#if 0 /* function removed */
-grib_context* grib_context_new(grib_context* parent)
-{
-    grib_context* c;
-#if GRIB_PTHREADS
-    pthread_mutexattr_t attr;
-#endif
+// Do we really need this?
+// grib_context* grib_context_new(grib_context* parent)
+// {
+//     grib_context* c;
+// #if GRIB_PTHREADS
+//     pthread_mutexattr_t attr;
+// #endif
 
-    if (!parent) parent=grib_context_get_default();
+//     if (!parent) parent=grib_context_get_default();
 
-    GRIB_MUTEX_INIT_ONCE(&once,&init);
-    GRIB_MUTEX_LOCK(&(parent->mutex));
+//     GRIB_MUTEX_INIT_ONCE(&once,&init);
+//     GRIB_MUTEX_LOCK(&(parent->mutex));
 
-    c = (grib_context*)grib_context_malloc_clear_persistent(&default_grib_context,sizeof(grib_context));
+//     c = (grib_context*)grib_context_malloc_clear_persistent(&default_grib_context,sizeof(grib_context));
 
-    c->inited              = default_grib_context.inited;
-    c->debug               = default_grib_context.debug;
+//     c->inited              = default_grib_context.inited;
+//     c->debug               = default_grib_context.debug;
 
-    c->real_mode           = default_grib_context.real_mode;
+//     c->real_mode           = default_grib_context.real_mode;
 
-    c->free_mem            = default_grib_context.free_mem;
-    c->alloc_mem           = default_grib_context.alloc_mem;
+//     c->free_mem            = default_grib_context.free_mem;
+//     c->alloc_mem           = default_grib_context.alloc_mem;
 
-    c->free_persistent_mem = default_grib_context.free_persistent_mem;
-    c->alloc_persistent_mem= default_grib_context.alloc_persistent_mem;
+//     c->free_persistent_mem = default_grib_context.free_persistent_mem;
+//     c->alloc_persistent_mem= default_grib_context.alloc_persistent_mem;
 
-    c->read                = default_grib_context.read;
-    c->write               = default_grib_context.write;
-    c->tell                = default_grib_context.tell;
+//     c->read                = default_grib_context.read;
+//     c->write               = default_grib_context.write;
+//     c->tell                = default_grib_context.tell;
 
-    c->output_log          = default_grib_context.output_log;
-    c->print               = default_grib_context.print    ;
-    c->user_data           = default_grib_context.user_data;
-    c->def_files           = default_grib_context.def_files;
-    c->lists               = default_grib_context.lists;
+//     c->output_log          = default_grib_context.output_log;
+//     c->print               = default_grib_context.print    ;
+//     c->user_data           = default_grib_context.user_data;
+//     c->def_files           = default_grib_context.def_files;
+//     c->lists               = default_grib_context.lists;
 
-#if GRIB_PTHREADS
-    pthread_mutexattr_settype(&attr,PTHREAD_MUTEX_RECURSIVE);
-    pthread_mutex_init(&mutex_c,&attr);
-    pthread_mutexattr_destroy(&attr);
-#endif
+// #if GRIB_PTHREADS
+//     pthread_mutexattr_settype(&attr,PTHREAD_MUTEX_RECURSIVE);
+//     pthread_mutex_init(&mutex_c,&attr);
+//     pthread_mutexattr_destroy(&attr);
+// #endif
+
+//     GRIB_MUTEX_UNLOCK(&(parent->mutex));
+//     return c;
+// }
 
-    GRIB_MUTEX_UNLOCK(&(parent->mutex));
-    return c;
-}
-#endif /* function removed */
 
 /* GRIB-235: Resolve path to expand symbolic links etc */
 /* Note: return value is allocated. Client has to free */
@@ -837,7 +839,6 @@ void grib_context_reset(grib_context* c)
 
 void grib_context_delete(grib_context* c)
 {
-    size_t i = 0;
     if (!c)
         c = grib_context_get_default();
 
@@ -849,7 +850,7 @@ void grib_context_delete(grib_context* c)
     if (c != &default_grib_context)
         grib_context_free_persistent(&default_grib_context, c);
 
-    for(i=0; i<MAX_NUM_HASH_ARRAY; ++i)
+    for(size_t i=0; i<MAX_NUM_HASH_ARRAY; ++i)
         c->hash_array[i] = NULL;
     c->hash_array_count = 0;
     grib_itrie_delete(c->hash_array_index);
@@ -904,9 +905,7 @@ void* grib_context_malloc_persistent(const grib_context* c, size_t size)
 {
     void* p = c->alloc_persistent_mem(c, size);
     if (!p) {
-        grib_context_log(c, GRIB_LOG_FATAL,
-                         "grib_context_malloc_persistent: error allocating %zu bytes", size);
-        Assert(0);
+        grib_context_log(c, GRIB_LOG_FATAL, "%s: error allocating %zu bytes", __func__, size);
     }
     return p;
 }
@@ -937,8 +936,7 @@ void* grib_context_malloc(const grib_context* c, size_t size)
     else
         p = c->alloc_mem(c, size);
     if (!p) {
-        grib_context_log(c, GRIB_LOG_FATAL, "grib_context_malloc: error allocating %zu bytes", size);
-        Assert(0);
+        grib_context_log(c, GRIB_LOG_FATAL, "%s: error allocating %zu bytes", __func__, size);
     }
     return p;
 }
@@ -950,7 +948,7 @@ void* grib_context_realloc(const grib_context* c, void* p, size_t size)
         c = grib_context_get_default();
     q = c->realloc_mem(c, p, size);
     if (!q) {
-        grib_context_log(c, GRIB_LOG_FATAL, "grib_context_realloc: error allocating %zu bytes", size);
+        grib_context_log(c, GRIB_LOG_FATAL, "%s: error allocating %zu bytes", __func__, size);
         return NULL;
     }
     return q;
@@ -985,7 +983,7 @@ void* grib_context_buffer_malloc(const grib_context* c, size_t size)
     else
         p = c->alloc_buffer_mem(c, size);
     if (!p) {
-        grib_context_log(c, GRIB_LOG_FATAL, "grib_context_buffer_malloc: error allocating %zu bytes", size);
+        grib_context_log(c, GRIB_LOG_FATAL, "%s: error allocating %zu bytes", __func__, size);
         return NULL;
     }
     return p;
@@ -1003,7 +1001,7 @@ void* grib_context_buffer_realloc(const grib_context* c, void* p, size_t size)
 {
     void* q = c->realloc_buffer_mem(c, p, size);
     if (!q) {
-        grib_context_log(c, GRIB_LOG_FATAL, "grib_context_buffer_realloc: error allocating %zu bytes", size);
+        grib_context_log(c, GRIB_LOG_FATAL, "%s: error allocating %zu bytes", __func__, size);
         return NULL;
     }
     return q;
@@ -1065,17 +1063,15 @@ void grib_context_log(const grib_context* c, int level, const char* fmt, ...)
             level = level & ~GRIB_LOG_PERROR;
 
             /* #if HAS_STRERROR */
-#if 1
             strcat(msg, " (");
             strcat(msg, strerror(errsv));
             strcat(msg, ")");
-#else
-            if (errsv > 0 && errsv < sys_nerr) {
-                strcat(msg, " (");
-                strcat(msg, sys_errlist[errsv]);
-                strcat(msg, " )");
-            }
-#endif
+
+            // if (errsv > 0 && errsv < sys_nerr) {
+            //     strcat(msg, " (");
+            //     strcat(msg, sys_errlist[errsv]);
+            //     strcat(msg, " )");
+            // }
         }
 
         if (c->output_log)
